<?php

namespace BlueSpice\UserSidebar\Component\SimpleCard;

use IContextSource;
use MediaWiki\MediaWikiServices;
use MWStake\MediaWiki\Component\CommonUserInterface\Component\Literal;
use MWStake\MediaWiki\Component\CommonUserInterface\Component\SimpleCard;
use MWStake\MediaWiki\Component\CommonUserInterface\Component\SimpleCardHeader;
use MWStake\MediaWiki\Component\CommonUserInterface\Component\SimpleLinklistGroupFromArray;
use Sanitizer;

class PersonalMenu extends SimpleCard {

	/**
	 * @var string
	 */
	protected $section = null;

	/**
	 * @var array
	 */
	protected $links = null;

	/**
	 * @var IContextSource
	 */
	protected $context = null;

	/**
	 * @param string $section
	 * @param array $links
	 */
	public function __construct( string $section, array $links ) {
		$this->section = $section;
		$this->links = $links;
		parent::__construct( [] );
	}

	/**
	 * @inheritDoc
	 */
	public function getId(): string {
		$id = Sanitizer::escapeIdForAttribute( $this->section );
		return "umcus-$id";
	}

	/**
	 * @inheritDoc
	 */
	public function getContainerClasses(): array {
		return [ 'card-mn' ];
	}

	/**
	 *
	 * @param IContextSource $context
	 * @return bool
	 */
	public function shouldRender( IContextSource $context ): bool {
		$this->context = $context;
		return !empty( $this->links );
	}

	/**
	 * @inheritDoc
	 */
	public function getSubComponents(): array {
		$id = strtolower( Sanitizer::escapeIdForAttribute( $this->section ) );
		$services = MediaWikiServices::getInstance();
		$linkFormatter = $services->getService( 'MWStakeLinkFormatter' );
		return [
			new SimpleCardHeader( [
				'id' => "umcus-menu-head-$id",
				'classes' => [ 'menu-title' ],
				'items' => [
					new Literal(
						"umcus-menu-title-$id",
						htmlspecialchars( $this->section )
					)
				]
			] ),
			new SimpleLinklistGroupFromArray( [
				'id' => "umcus-manu-$id-linkgroup",
				'classes' => [ 'menu-card-body', 'menu-list', 'll-dft' ],
				'links' => $linkFormatter->formatLinks( $this->links ),
				'aria' => [
					'labelledby' => "umcus-menu-head-$id"
				]
			] ),
		];
	}
}
